@@ -10,6 +10,23 @@ const appSvc = Cc["@mozilla.org/appshell/appShellService;1"]
 
 const gmSvcFilename = Components.stack.filename;
 
+const maxJSVersion = (function getMaxJSVersion() {
+  var appInfo = Components
+      .classes["@mozilla.org/xre/app-info;1"]
+      .getService(Components.interfaces.nsIXULAppInfo);
+  var versionChecker = Components
+      .classes["@mozilla.org/xpcom/version-comparator;1"]
+      .getService(Components.interfaces.nsIVersionComparator);
+
+  // Firefox 3.5 and higher supports 1.8.
+  if (versionChecker.compare(appInfo.version, "3.5") >= 0) {
+    return "1.8";
+  }
+
+  // Everything else supports 1.6.
+  return "1.6";
+})();
+
 function alert(msg) {
   Cc["@mozilla.org/embedcomp/prompt-service;1"]
     .getService(Ci.nsIPromptService)
@@ -50,7 +67,6 @@ var greasemonkeyService = {
     return this._config;
   },
   browserWindows: [],
-  updater: null,
 
 
   // nsISupports
@@ -112,15 +128,6 @@ var greasemonkeyService = {
     if (scripts.length > 0) {
       this.injectScripts(scripts, href, unsafeWin, chromeWin);
     }
-
-    // Need to wait until well after startup for prefs store and extension
-    // manager to be initialized. First page load is a convenient place.
-    if (!this.updater) {
-      // Note: the param to this has to match the extension ID in install.rdf
-      this.updater = new ExtensionUpdater(
-          "{e4a8a97b-f2ed-450b-b12d-ee082ba24781}");
-      this.updater.updatePeriodically();
-    }
   },
 
 
@@ -134,8 +141,6 @@ var greasemonkeyService = {
     loader.loadSubScript("chrome://greasemonkey/content/convert2RegExp.js");
     loader.loadSubScript("chrome://greasemonkey/content/miscapis.js");
     loader.loadSubScript("chrome://greasemonkey/content/xmlhttprequester.js");
-    loader.loadSubScript("chrome://greasemonkey/content/updater.js");
-    //loggify(this, "GM_GreasemonkeyService");
   },
 
   shouldLoad: function(ct, cl, org, ctx, mt, ext) {
@@ -167,11 +172,11 @@ var greasemonkeyService = {
 
       if (!this.ignoreNextScript_) {
         if (!this.isTempScript(cl)) {
-          var winWat = Cc["@mozilla.org/embedcomp/window-watcher;1"]
-            .getService(Ci.nsIWindowWatcher);
-
-          if (winWat.activeWindow && winWat.activeWindow.GM_BrowserUI) {
-            winWat.activeWindow.GM_BrowserUI.startInstallScript(cl);
+          var win = Cc['@mozilla.org/appshell/window-mediator;1']
+            .getService(Ci.nsIWindowMediator)
+            .getMostRecentWindow("navigator:browser");
+          if (win && win.GM_BrowserUI) {
+            win.GM_BrowserUI.startInstallScript(cl);
             ret = Ci.nsIContentPolicy.REJECT_REQUEST;
           }
         }
@@ -238,7 +243,8 @@ var greasemonkeyService = {
 
       storage = new GM_ScriptStorage(script);
       xmlhttpRequester = new GM_xmlhttpRequester(unsafeContentWin,
-                                                 appSvc.hiddenDOMWindow);
+                                                 appSvc.hiddenDOMWindow,
+                                                 url);
       resources = new GM_Resources(script);
 
       sandbox.window = safeWin;
@@ -258,7 +264,7 @@ var greasemonkeyService = {
       sandbox.GM_listValues = GM_hitch(storage, "listValues");
       sandbox.GM_getResourceURL = GM_hitch(resources, "getResourceURL");
       sandbox.GM_getResourceText = GM_hitch(resources, "getResourceText");
-      sandbox.GM_openInTab = GM_hitch(this, "openInTab", unsafeContentWin);
+      sandbox.GM_openInTab = GM_hitch(this, "openInTab", safeWin, chromeWin);
       sandbox.GM_xmlhttpRequest = GM_hitch(xmlhttpRequester,
                                            "contentStartRequest");
       sandbox.GM_registerMenuCommand = GM_hitch(this,
@@ -279,7 +285,7 @@ var greasemonkeyService = {
         requires.push(contents);
         offset += lineCount;
         offsets.push(offset);
-      })
+      });
       script.offsets = offsets;
 
       var scriptSrc = "\n" + // error line-number calculations depend on these
@@ -297,6 +303,10 @@ var greasemonkeyService = {
 
   registerMenuCommand: function(unsafeContentWin, commandName, commandFunc,
                                 accelKey, accelModifiers, accessKey) {
+    if (!GM_apiLeakCheck("GM_registerMenuCommand")) {
+      return;
+    }
+
     var command = {name: commandName,
                    accelKey: accelKey,
                    accelModifiers: accelModifiers,
@@ -309,37 +319,58 @@ var greasemonkeyService = {
     }
   },
 
-  openInTab: function(unsafeContentWin, url) {
-    var unsafeTop = new XPCNativeWrapper(unsafeContentWin, "top").top;
-
-    for (var i = 0; i < this.browserWindows.length; i++) {
-      this.browserWindows[i].openInTab(unsafeTop, url);
+  openInTab: function(safeContentWin, chromeWin, url) {
+    if (!GM_apiLeakCheck("GM_openInTab")) {
+      return undefined;
     }
+
+    var info = Cc["@mozilla.org/xre/app-info;1"]
+      .getService(Components.interfaces.nsIXULAppInfo);
+    if (parseFloat(info.version, 10) < 3.0) {
+      // Pre FF 3.0 wants the URL as the second argument.
+      var newTab = chromeWin.openNewTabWith(
+        url, safeContentWin.document.location.href, null, null, null, null);
+    } else {
+      // Post FF 3.0 wants the document as the second argument.
+      var newTab = chromeWin.openNewTabWith(
+        url, safeContentWin.document, null, null, null, null);
+    }
+
+    // Source:
+    // http://mxr.mozilla.org/mozilla-central/source/browser/base/content/browser.js#4448
+    var newWindow = chromeWin.gBrowser
+      .getBrowserForTab(newTab)
+      .docShell
+      .QueryInterface(Ci.nsIInterfaceRequestor)
+      .getInterface(Ci.nsIDOMWindow);
+    return newWindow;
   },
 
   evalInSandbox: function(code, codebase, sandbox, script) {
     if (!(Components.utils && Components.utils.Sandbox)) {
       var e = new Error("Could not create sandbox.");
       GM_logError(e, 0, e.fileName, e.lineNumber);
-    } else {
+      return true;
+    }
+    try {
+      // workaround for https://bugzilla.mozilla.org/show_bug.cgi?id=307984
+      var lineFinder = new Error();
+      Components.utils.evalInSandbox(code, sandbox, maxJSVersion);
+    } catch (e) { // catches errors while running the script code
       try {
-        // workaround for https://bugzilla.mozilla.org/show_bug.cgi?id=307984
-        var lineFinder = new Error();
-        Components.utils.evalInSandbox(code, sandbox);
-      } catch (e) {
-        if ("return not in function" == e.message) // pre-0.8 GM compat:
-          return false; // this script depends on the function enclosure
+        if (e && "return not in function" == e.message)
+          return false; // means this script depends on the function enclosure
 
         // try to find the line of the actual error line
-        var line = e.lineNumber;
+        var line = e && e.lineNumber;
         if (4294967295 == line) {
           // Line number is reported as max int in edge cases.  Sometimes
           // the right one is in the "location", instead.  Look there.
           if (e.location && e.location.lineNumber) {
             line = e.location.lineNumber;
           } else {
-            // Reporting max int is useless, if we couldn't find it in location
-            // either, forget it.  Value of 0 isn't shown in the console.
+            // Reporting maxint is useless, if we couldn't find it in location
+            // either, forget it.  A value of 0 isn't shown in the console.
             line = 0;
           }
         }
@@ -360,6 +391,8 @@ var greasemonkeyService = {
             0
           );
         }
+      } catch (e) { // catches errors we cause trying to inform the user
+        // Do nothing. More importantly: don't stop script incovation sequence.
       }
     }
     return true; // did not need a (function() {...})() enclosure.
@@ -387,6 +420,10 @@ var greasemonkeyService = {
   },
 
   getFirebugConsole: function(unsafeContentWin, chromeWin) {
+    // If we can't find this object, there's no chance the rest of this
+    // function will work.
+    if ('undefined'==typeof chromeWin.Firebug) return null;
+
     try {
       chromeWin = chromeWin.top;
       var fbVersion = parseFloat(chromeWin.Firebug.version, 10);
@@ -394,6 +431,12 @@ var greasemonkeyService = {
       var fbContext = chromeWin.TabWatcher &&
         chromeWin.TabWatcher.getContextByWindow(unsafeContentWin);
 
+      // Firebug 1.4 will give no context, when disabled for the current site.
+      // We can't run that way.
+      if ('undefined'==typeof fbContext) {
+        return null;
+      }
+
       function findActiveContext() {
         for (var i=0; i<fbContext.activeConsoleHandlers.length; i++) {
           if (fbContext.activeConsoleHandlers[i].window == unsafeContentWin) {
@@ -430,7 +473,7 @@ var greasemonkeyService = {
         dummyElm.parentNode.removeChild(dummyElm);
 
         return fbContext.consoleHandler.pop().handler;
-      } else if (1.3 == fbVersion || 1.4 == fbVersion) {
+      } else if (fbVersion >= 1.3) {
         fbConsole.injector.attachIfNeeded(fbContext, unsafeContentWin);
         return findActiveContext();
       }
@@ -444,8 +487,6 @@ var greasemonkeyService = {
 
 greasemonkeyService.wrappedJSObject = greasemonkeyService;
 
-//loggify(greasemonkeyService, "greasemonkeyService");
-
 
 
 /**
@@ -509,6 +550,3 @@ Factory.createInstance = function(outer, iid) {
 function NSGetModule(compMgr, fileSpec) {
   return Module;
 }
-
-//loggify(Module, "greasemonkeyService:Module");
-//loggify(Factory, "greasemonkeyService:Factory");
